/*
   AwcrMAN is released under GPL licence.
   Check file GPL.txt for more informations.
*/

OPT OSVERSION=37, REG=3

MODULE 'dos/dos', 'dos/datetime'

ENUM OK, NOARGS, ERROR, NOAWCR, NEWAWCR, OLDAWCR, BADTYPE, ALREADYNEW

ENUM ARG_FILE, ARG_UPDATE, ARG_INFO, ARG_LIST, ARG_URL

CONST AWCR21=2, AWCR30=3, KNOWN_AWCR=3, MIN_AWCR=2

OBJECT cb
  check:LONG
  version:LONG
  last:LONG
ENDOBJECT

OBJECT cf
  number:LONG
  date:LONG
  expire:LONG
  cachedate:LONG
  size:LONG
  type:INT
  urllen:INT
ENDOBJECT

DEF infileh=NIL, infilel=0, buffer=NIL, info:fileinfoblock,
   cbase:cb, cfile:cf, dt:datetime,
   date[80]:ARRAY, time[80]:ARRAY, txtdate[255]:STRING,
   args:PTR TO LONG, rd=NIL


PROC openfile()
  infileh:=Open(args[ARG_FILE], OLDFILE)
  IF (infileh = NIL) THEN Raise(ERROR)

  IF (ExamineFH(infileh, info) = NIL) THEN Raise(ERROR)

  infilel:=info.size
  IF (infilel = 0)
    Raise(ERROR)
  ELSEIF (infilel < SIZEOF cb)
    Raise(NOAWCR)
  ENDIF

  buffer:=New(infilel)
  IF (buffer = NIL) THEN Raise(ERROR)

  IF (Read(infileh, buffer, infilel) = -1) THEN Raise(ERROR)
ENDPROC

PROC writefile()
  DEF outfileh=NIL

  outfileh:=Open(args[ARG_UPDATE], NEWFILE)
  IF (outfileh = NIL) THEN Raise(ERROR)

  IF Write(outfileh, buffer, infilel) = FALSE THEN Raise(ERROR)
  Close(outfileh)
ENDPROC


PROC main() HANDLE
  VOID '$VER: AwcrMAN 1.0 (20.7.97) by Gabriele Favrin (gabriele@favrin.net)'

  args:=[0,0,0,0,0,0]
  rd:=ReadArgs('FILE/A,UPDATE/K,INFO/S,LIST/S,URL/S', args, NIL)
  WriteF('AwcrMAN 1.0 by Gabriele Favrin (gabriele@favrin.net)\n\n')

  IF rd = NIL THEN Raise(NOARGS)

  IF args[ARG_UPDATE]
    args[ARG_INFO]:=FALSE
    args[ARG_URL]:=FALSE
    args[ARG_LIST]:=FALSE
  ENDIF

  openfile()
  checkAWCR()

  IF (args[ARG_INFO] = FALSE) THEN meltAWCR()


EXCEPT DO
  IF (exception = ERROR)
    PrintFault(IoErr(), NIL)
  ELSEIF (exception = NOARGS)
    WriteF('\s', {help})
  ELSEIF (exception = NOAWCR)
    WriteF('Not an AWCR file.\n')
  ELSEIF (exception = NEWAWCR)
    WriteF('You need a newer version of AwcrMAN!\n')
  ELSEIF (exception = OLDAWCR)
    WriteF('AWCR format too old.\n')
  ELSEIF (exception = BADTYPE)
    WriteF('Invalid file type (AWCR file damaged?)\n')
  ELSEIF (exception = ALREADYNEW)
    WriteF('This is already an AWeb 3.0+ AWCR file!\n')
  ENDIF

  IF infileh THEN Close(infileh)
  IF rd      THEN FreeArgs(rd)

ENDPROC

PROC checkAWCR()
  cbase:=buffer

  IF cbase.check <> "AWCR" THEN Raise(NOAWCR)

  IF (cbase.version > KNOWN_AWCR)
    Raise(NEWAWCR)
  ELSEIF (cbase.version < MIN_AWCR)
    Raise(OLDAWCR)
  ELSEIF (cbase.version = 3) AND (args[ARG_UPDATE])
    Raise(ALREADYNEW)
  ENDIF

  WriteF('AWCR version is \d\nSize of file is \d\nLast cache number is \d\n\n',
       cbase.version, infilel, cbase.last)

ENDPROC

PROC meltAWCR()
  DEF pos, flag, mtype, url, murl, files=0:REG, newtypes

  IF args[ARG_UPDATE]
    newtypes:=[0,0,2,3,0]
  ELSE
    IF (args[ARG_UPDATE])
      WriteF('Updating AWCR...\n')
    ELSEIF (args[ARG_LIST] = FALSE) AND (args[ARG_URL] = FALSE)
      WriteF('Counting files...\n')
    ENDIF
  ENDIF
  pos:=SIZEOF cb

  WHILE pos < infilel
    EXIT CtrlC() = TRUE

    INC files
    cfile:=buffer+pos
    pos:=pos+SIZEOF cf

    mtype:=buffer+pos
    pos:=pos+StrLen(mtype)+1

    url:=buffer+pos
    pos:=pos+StrLen(url)+1

    flag:=FALSE
    IF (cbase.version = AWCR30)
      IF (cfile.type = 1) OR (cfile.type = 2) THEN flag:=TRUE
    ELSE
      IF cfile.type = 3 THEN flag:=TRUE
    ENDIF

    IF flag
      murl:=buffer+pos
      pos:=pos+StrLen(murl)+1
    ELSE
      murl:=0
    ENDIF

    IF args[ARG_UPDATE]
      IF (cfile.type < 6) AND (cfile.type > -1)
        cfile.type:=newtypes[cfile.type]
      ELSE
        WriteF('type=\d files=\d\n', cfile.type, files)
        Raise(BADTYPE)
      ENDIF
    ELSE
      IF args[ARG_LIST]
        showcfile(mtype, url, murl)
      ELSEIF args[ARG_URL]
        WriteF('\s\n', url)
      ENDIF
    ENDIF

  ENDWHILE
  IF args[ARG_UPDATE]
    cbase.version:=3
    writefile()
    WriteF('Conversion done. File saved as \s\n', args[ARG_UPDATE])
  ELSE
    WriteF('\nFound \d files.\n', files)
  ENDIF

ENDPROC

PROC showcfile(mtype, url, murl)
  WriteF('\nNumber    = \d\n', cfile.number)
  WriteF('Date      = \s\n', showDateStamp(cfile.date))
  WriteF('Expires   = \s\n', showDateStamp(cfile.expire))
  WriteF('Cachedate = \s\n', showDateStamp(cfile.cachedate))

  WriteF('Size      = \d\n' +
       'Type      = \d\n' +
       'URL len   = \d\n' +
       'Mime Type = \s\n' +
       'URL       = \s\n',
       cfile.size, cfile.type, cfile.urllen,
       (IF mtype[] THEN mtype ELSE ''), url)

  IF murl THEN WriteF('Moved to  = \s\n', murl)
ENDPROC


PROC showDateStamp(awebdate)
  DEF a

  IF awebdate
    dt.stamp.days:=Div(awebdate, 86400)

    a:=awebdate/60
    dt.stamp.minute:=Mod(a, 60)

    a:=Mod(awebdate, 60)
    dt.stamp.tick:=(a * TICKS_PER_SECOND)

    dt.format:=FORMAT_DOS
    dt.strday:=NIL
    dt.strdate:=date
    dt.strtime:=time
    dt.flags:=DTF_FUTURE

    IF DateToStr(dt)
      StringF(txtdate, '\s \s', date, time)
      RETURN txtdate
    ENDIF
  ELSE
    RETURN 'not set'
  ENDIF

ENDPROC


help:
INCBIN 'awcrman.help'
CHAR 0
